<?php

namespace WPDesk\FiGTU;

use FIGTUVendor\WPDesk\Library\FlexibleInvoicesAbstracts\Documents\Document;
use FIGTUVendor\WPDesk\Persistence\PersistentContainer;
use FIGTUVendor\WPDesk\PluginBuilder\Plugin\Hookable;
use FIGTUVendor\WPDesk\View\Renderer\Renderer;

/**
 * Integrate with Flexible Invoices.
 *
 * @package WPDesk\FiGTU
 */
class Flexible_Invoices_Integration implements Hookable {

	const SELECT_ID = '_flexible_invoices_gtu_code';
	const META_KEY  = '_gtu_codes';


	private $renderer;

	/**
	 * @param Renderer $renderer
	 */
	public function __construct( Renderer $renderer ) {
		$this->renderer = $renderer;
	}

	/**
	 * Fires hooks.
	 *
	 * @since 1.0.0
	 */
	public function hooks() {
		add_action( 'woocommerce_product_options_general_product_data', array( $this, 'render_select' ) );
		add_action( 'woocommerce_process_product_meta', array( $this, 'save_product_meta_value' ), 10, 2 );
		add_action( 'fi/core/document/save', array( $this, 'save_document_post_meta' ), 10, 2 );
		add_action( 'fi/core/template/invoice/after_notes', array( $this, 'after_invoice_notes' ), 10, 1 );
		add_action( 'fi/core/layout/metabox/payment/after', array( $this, 'after_meta_box_payment' ), 10, 1 );
	}

	/**
	 * @return void
	 *
	 * @since 1.0.0
	 */
	public function render_select() {
		global $post;

		$options = [
			'0' => 'brak'
		];
		foreach ( $this->get_gtu_codes() as $codes ) {
			$options[ $codes['name'] ] = $codes['name'] . ' - ' . $codes['short_description'];
		}
		echo $this->renderer->render( 'select', [ 'options' => $options, 'post_id' => $post->ID ] );
	}

	/**
	 * @param int $product_id Product ID.
	 *
	 * @since 1.0.0
	 */
	public function save_product_meta_value( $product_id ) {
		if ( isset( $_POST[ self::SELECT_ID ] ) ) {
			$gtu_code = isset( $_POST[ self::SELECT_ID ] ) ? wp_unslash( $_POST[ self::SELECT_ID ] ) : '';
			update_post_meta( $product_id, self::SELECT_ID, $gtu_code );
		}
	}

	/**
	 * Save custom post meta for the document.
	 *
	 * @param Document            $document Document type.
	 * @param PersistentContainer $meta     Meta Container.
	 *
	 * @return bool
	 *
	 * @since 1.0.0
	 */
	public function save_document_post_meta( $document, $meta ) {
		$gtu_codes = [];
		$order     = wc_get_order( $document->get_order_id() );
		if ( ! $order ) {
			return false;
		}

		$items = $order->get_items( 'line_item' );
		if ( $items ) {
			foreach ( $items as $item ) {
				$codes       = $this->get_gtu_codes();
				$product_gtu = get_post_meta( $item->get_product_id(), self::SELECT_ID, true );
				if ( ! empty( $product_gtu ) && isset( $codes[ $product_gtu ] ) ) {
					$gtu_codes[] = $product_gtu;
				}
			}

			if ( ! empty( $gtu_codes ) ) {
				$meta->set( '_gtu_codes', $gtu_codes );
			}
		}

		return true;
	}

	/**
	 * @param Document $document
	 *
	 * @return void
	 */
	public function after_invoice_notes( $document ) {
		$gtu_codes = get_post_meta( $document->get_id(), self::META_KEY, true );
		if ( ! empty( $gtu_codes ) ) {
			echo $this->renderer->render( 'after-invoice-notes', [ 'gtu_codes' => $gtu_codes ] );
		}
	}

	/**
	 * @param Document $document
	 *
	 * @return void
	 */
	public function after_meta_box_payment( $document ) {
		$gtu_codes = get_post_meta( $document->get_id(), self::META_KEY, true );
		if ( ! empty( $gtu_codes ) ) {
			echo $this->renderer->render( 'meta-box-payment', [ 'gtu_codes' => $gtu_codes ] );
		}
	}

	/**
	 * @return array
	 *
	 * @since 1.0.0
	 */
	public function get_gtu_codes() {
		return [
			'GTU_01' => [
				'name'              => 'GTU_01',
				'short_description' => 'Dostawa napojów alkoholowych',
				'description'       => 'Dostawa napojów alkoholowych - alkoholu etylowego, piwa, wina, napojów fermentowanych i wyrobów pośrednich, w rozumieniu przepisów o podatku akcyzowym.'
			],
			'GTU_02' => [
				'name'              => 'GTU_02',
				'short_description' => 'Dostawa paliw silnikowych',
				'description'       => 'Dostawa towarów, o których mowa w art. 103 ust. 5aa ustawy.'
			],
			'GTU_03' => [
				'name'              => 'GTU_03',
				'short_description' => 'Dostawa oleju opałowego, olejów smarownych oraz smarów plastycznych itp.',
				'description'       => 'Dostawa oleju opałowego w rozumieniu przepisów o podatku akcyzowym oraz olejów smarowych, pozostałych olejów o kodach CN od 2710 19 71 do 2710 19 99, z wyłączeniem wyrobów o kodzie CN 2710 19 85 (oleje białe, parafina ciekła) oraz smarów plastycznych zaliczanych do kodu CN 2710 19 99, olejów smarowych o kodzie CN 2710 20 90, preparatów smarowych objętych pozycją CN 3403, z wyłączeniem smarów plastycznych objętych tą pozycją.'
			],
			'GTU_04' => [
				'name'              => 'GTU_04',
				'short_description' => 'Dostawa wyrobów tytoniowych oraz wyborów nowatorskich',
				'description'       => 'Dostawa wyrobów tytoniowych, suszu tytoniowego, płynu do papierosów elektronicznych i wyrobów nowatorskich, w rozumieniu przepisów o podatku akcyzowym.'
			],
			'GTU_05' => [
				'name'              => 'GTU_05',
				'short_description' => 'Dostawa odpadów',
				'description'       => 'Dostawa odpadów - wyłącznie określonych w poz. 79-91 załącznika nr 15 do ustawy.'
			],
			'GTU_06' => [
				'name'              => 'GTU_06',
				'short_description' => 'Dostawa urządzeń elektronicznych oraz części i materiałów do nich',
				'description'       => 'Dostawa urządzeń elektronicznych oraz części i materiałów do nich, wyłącznie określonych w poz. 7-9, 59-63, 65, 66, 69 i 94-96 załącznika nr 15 do ustawy.'
			],
			'GTU_07' => [
				'name'              => 'GTU_07',
				'short_description' => 'Dostawa pojazdów oraz części samochodowych',
				'description'       => 'Dostawa pojazdów oraz części samochodowych o kodach wyłącznie CN 8701 - 8708 oraz CN 8708 10.'
			],
			'GTU_08' => [
				'name'              => 'GTU_08',
				'short_description' => 'Dostawa metali szlachetnych oraz nieszlachetnych',
				'description'       => 'Dostawa metali szlachetnych oraz nieszlachetnych - wyłącznie określonych w poz. 1-3 załącznika nr 12 do ustawy oraz w poz. 12-25, 33-40, 45, 46, 56 i 78 załącznika nr 15 do ustawy.'
			],
			'GTU_09' => [
				'name'              => 'GTU_09',
				'short_description' => 'Dostawa leków oraz wyrobów medycznych',
				'description'       => 'Dostawa leków oraz wyrobów medycznych - produktów leczniczych, środków spożywczych specjalnego przeznaczenia żywieniowego oraz wyrobów medycznych, objętych obowiązkiem zgłoszenia, o którym mowa w art. 37av ust. 1 ustawy z dnia 6 września 2001 r. - Prawo farmaceutyczne (Dz. U. z 2019 r. poz. 499, z późn. zm.).'
			],
			'GTU_10' => [
				'name'              => 'GTU_10',
				'short_description' => 'Dostawa budynków, budowli i gruntów.',
				'description'       => 'Dostawa budynków, budowli i gruntów.'
			],
			'GTU_11' => [
				'name'              => 'GTU_11',
				'short_description' => 'Świadczenie usług w zakresie przenoszenia uprawnień do emisji gazów cieplarnianych',
				'description'       => 'Dostawa o których mowa w ustawie z dnia 12 czerwca 2015 r. o systemie handlu uprawnieniami do emisji gazów cieplarnianych (Dz. U. z 2018 r. poz. 1201 i 2538 oraz z 2019 r. poz. 730, 1501 i 1532)..'
			],
			'GTU_12' => [
				'name'              => 'GTU_12',
				'short_description' => 'Świadczenie usług o charakterze niematerialnym',
				'description'       => 'Świadczenie usług o charakterze niematerialnym - wyłącznie: doradczych, księgowych, prawnych, zarządczych, szkoleniowych, marketingowych, firm centralnych (head offices), reklamowych, badania rynku i opinii publicznej, w zakresie badań naukowych i prac rozwojowych.'
			],
			'GTU_13' => [
				'name'              => 'GTU_13',
				'short_description' => 'Świadczenie usług transportowych i gospodarki magazynowej',
				'description'       => 'Świadczenie usług transportowych i gospodarki magazynowej - Sekcja H PKWiU 2015 symbol ex 49.4, ex 52.1.'
			]
		];
	}
}
